package ch.zhaw.gartenverwaltung.taskList;

import ch.zhaw.gartenverwaltung.Config;
import ch.zhaw.gartenverwaltung.io.HardinessZoneNotSetException;
import ch.zhaw.gartenverwaltung.io.JsonTaskDatabase;
import ch.zhaw.gartenverwaltung.io.PlantDatabase;
import ch.zhaw.gartenverwaltung.io.TaskDatabase;
import ch.zhaw.gartenverwaltung.types.*;

import java.io.IOException;
import java.time.LocalDate;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

public class TaskListModel {
    private TaskDatabase taskDatabase;
    private PlantDatabase plantDatabase;

    /**
     * Comparators to create sorted Task List
     */
    static final Comparator<Task> sortByStartDate = Comparator.comparing(Task::getStartDate);

    public TaskListModel(){
        taskDatabase = new JsonTaskDatabase();
    }

    /**
     * Constructor to create Database Objects.
     */
    public TaskListModel(TaskDatabase taskDatabase, PlantDatabase plantDatabase) {
        this.taskDatabase = taskDatabase;
        this.plantDatabase = plantDatabase;
    }

    /**
     * Method to save a new Task to Task Database
     * @param task the Task to save
     * @throws IOException If the database cannot be accessed
     */
    public void addTask(Task task) throws IOException {
        taskDatabase.saveTask(task);
    }

    /**
     * Method to add all Tasks for a new crop
     * @param crop the crop which is added
     * @throws PlantNotFoundException if the plantId in the crop doesn't exist in Plant Database
     * @throws HardinessZoneNotSetException If there is no Hardiness Zone Set in Plant Database
     * @throws IOException If the database cannot be accessed
     */
    public void planTasksForCrop(Crop crop) throws PlantNotFoundException, HardinessZoneNotSetException, IOException {
        Plant plant = plantDatabase.getPlantById(Config.getCurrentHardinessZone(), crop.getPlantId()).orElseThrow(PlantNotFoundException::new);
        for (GrowthPhase growthPhase : plant.lifecycle()) {
            for (TaskTemplate taskTemplate : growthPhase.taskTemplates()) {
                addTask(taskTemplate.generateTask(crop.getStartDate(), crop.getCropId().orElse(0L)));
            }
        }
    }

    /**
     * Method to remove all Tasks for a specific Crop
     * @param cropId The crop which is removed
     */
    public void removeTasksForCrop(long cropId) throws IOException {
        taskDatabase.removeTasksForCrop(cropId);
    }

    public void removeTask(Task task) throws IOException {
        taskDatabase.removeTask(task);
    }

    public List<Task> getTaskList() throws IOException {
        return getFilteredTaskList(LocalDate.MIN, LocalDate.MAX);
    }

    public List<Task> getFutureTasks() throws IOException {
        return getFilteredTaskList(LocalDate.now(), LocalDate.MAX);
    }

    public List<Task> getPastTasks() throws IOException {
        return getFilteredTaskList(LocalDate.MIN, LocalDate.now());
    }

    public List<Task>[] getTasksUpcomingWeek() throws IOException {
        List<Task>[] listArray = new List[7];
        for(int i = 0; i < 7; i++) {
            LocalDate date = LocalDate.now().plusDays(i);
            listArray[i] = taskDatabase.getTaskList(date, date);
        }
        return listArray;
    }

    public List<Task> getFilteredTaskList(LocalDate start, LocalDate end) throws IOException {
        return getSortedTaskList(taskDatabase.getTaskList(start, end), sortByStartDate);
    }

    private List<Task> getSortedTaskList(List<Task> taskList, Comparator<Task> comparator) {
        return taskList.stream().sorted(comparator).collect(Collectors.toList());
    }
}
